/****************************************************************
**
** Implementation of CIGLWIdget class
**
****************************************************************/


#include "CIGLWidget.h"
#include <QMessageBox.h>
#include <QImage.h>
#include <vcg/CI3ds.h>

CI3ds Obj3ds;

CIGLWidget::CIGLWidget( QWidget *parent, const char *name ): QGLWidget(parent, name)
{
}
	
void CIGLWidget::initializeGL()
{   
  // Set up the rendering context, define display lists etc.:
  glClearColor( 0.0, 1.0, 1.0, 0.0 );
	float spec[4]={.4,.4,.4,1};
	float shin = 1;
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR , spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS , shin);
	
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_COLOR_MATERIAL);
	
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	// Disabilita lo shading di gouraud per le facce dei poligoni disegnati
	glShadeModel(GL_SMOOTH);
	
	// Se scommentato disegna solo in wireframe
	//glPolygonMode(GL_FRONT,GL_LINE);
	tb.Init(size().width(),size().height());
	rot.SetIdentity();

	QImage tt; 	tt.load("Texture.png");
	QImage tx = QGLWidget::convertToGLFormat ( tt);

	GLuint texName;
	glGenTextures(1, &texName);
	glBindTexture(GL_TEXTURE_2D, texName);
	glTexImage2D( GL_TEXTURE_2D, 0, 3, tx.width(), tx.height(), 0,
                      GL_RGBA, GL_UNSIGNED_BYTE, tx.bits() );
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, tx.width(), tx.height(), GL_RGBA, GL_UNSIGNED_BYTE, tx.bits() );
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	Obj3ds.Open("knot3.3DS");
	//Obj3ds.Open("bea250a.3ds");
}

void CIGLWidget::resizeGL( int w, int h )
{
  // setup viewport, projection etc.:
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(45,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
	tb.Resize( w,h );

}

void CIGLWidget::paintGL()
{
	glMatrixMode (GL_TEXTURE);
	glLoadIdentity();
	glScalef(5,3,1);
	glMatrixMode (GL_MODELVIEW);
	glClear(GL_COLOR_BUFFER_BIT |GL_DEPTH_BUFFER_BIT  );
	glLoadIdentity();
	gluLookAt(2,5,12,0,0,0,0,1,0);
	glColor3f(1,.4,.4);
//	QString tmp;
//	renderText(lx,ly,tmp.sprintf("Mouse pressed on %i %i",lx,ly));
	glMultMatrix(rot);
	ring->m.Draw();
	glScalef(.07,.07,.07);
	//glDisable(GL_TEXTURE_2D);
	static int cnt=0;
	cnt=(cnt+1)%Obj3ds.FrameNum();
	Obj3ds.Draw(cnt);
}

void CIGLWidget::mousePressEvent(QMouseEvent*e)
{
  tb.MouseDown( e->x(), e->y(), 0 );
	update();
}

void CIGLWidget::mouseMoveEvent(QMouseEvent*e)
{
  tb.CalcRotMatrix( rot, e->x(), e->y());
	update();
}

void CIGLWidget::mouseReleaseEvent(QMouseEvent*e)
{
  tb.MouseUp( e->x(), e->y());
	update();
}
